require 'spec_helper'

describe 'puppet_enterprise::console::config' do
  before :each do
    @facter_facts = {
      'osfamily'          => 'Debian',
      'operatingsystem'   => 'Debian',
      'lsbmajdistrelease' => '6',
      'puppetversion'     => '3.6.2 (Puppet Enterprise 3.3.0)',
      'is_pe'             => 'true',
      'fqdn'              => 'console.rspec',
      'clientcert'        => 'awesomecert',
      'pe_concat_basedir'    => '/tmp/file',
    }

    @params = { 'secret_token'            => 'totally secret',
                'master_host'             => 'master.rspec',
                'console_server_certname' => 'console-client-rspec',
                'classifier_host'         => 'classifier.rspec',
                'ca_host'                 => 'ca.rspec',
                'delayed_job_workers'     => 2,
                'puppetdb_host'           => 'puppetdb.rspec',
                'puppetdb_port'           => '1234',
                'disable_live_management' => true,
                'console_client_certname' => 'dashboard-cn' }
  end

  let(:facts) { @facter_facts }
  let(:params) { @params }
  let(:certdir) { '/opt/puppet/share/puppet-dashboard/certs' }
  let(:dashboard_cert_whitelist) { '/etc/puppetlabs/puppet-dashboard/dashboard-certificate-whitelist' }

  describe "managing cert whitelist file" do
    it { should contain_file(dashboard_cert_whitelist) }
  end

  describe "managing settings.yml" do
    let(:file) { '/etc/puppetlabs/puppet-dashboard/settings.yml' }
    it { should contain_file(file).with(:owner => 'puppet-dashboard',
                                        :group => 'puppet-dashboard',
                                        :mode => '0640') }

    it { should have_yaml_file(file, 'ca_server' => 'ca.rspec') }
    it { should have_yaml_file(file, 'inventory_server' => 'master.rspec') }
    it { should have_yaml_file(file, 'file_bucket_server' => 'master.rspec') }
    it { should have_yaml_file(file, 'nc_api_url' => 'https://classifier.rspec:4433/classifier-api') }
    it { should have_yaml_file(file, 'secret_token' => 'totally secret') }
    it { should have_yaml_file(file, 'disable_live_management' => true) }
    it { should have_yaml_file(file, 'certificate_whitelist' => dashboard_cert_whitelist ) }
  end

  describe "not managing settings.yml" do
    let(:file) { '/etc/puppetlabs/puppet-dashboard/settings.yml' }
    before :each do
      @params['secret_token'] = ''
    end
    it { should_not contain_file(file) }
  end

  describe "managing puppetdashboard.conf" do
    let(:file) { '/etc/puppetlabs/httpd/conf.d/puppetdashboard.conf' }
    it { should contain_file(file).with(:owner => 'root',
                                        :group => 'root',
                                        :mode => '0644') }


    it { should contain_file(file).with_content(%r[SSLCertificateFile\s+#{certdir}/console-client-rspec.cert.pem]) }
    it { should contain_file(file).with_content(%r[SSLCertificateKeyFile\s+#{certdir}/console-client-rspec.private_key.pem]) }
    it { should contain_file(file).with_content(%r[SSLCertificateChainFile\s+/etc/puppetlabs/puppet/ssl/certs/ca.pem]) }
    it { should contain_file(file).with_content(%r[SSLCACertificateFile\s+/etc/puppetlabs/puppet/ssl/certs/ca.pem]) }

    it { should_not contain_file(file).with_content(%r[SetEnv\s+LM_DISCOVERY_TIMEOUT\s+]) }
    it { should_not contain_file(file).with_content(%r[SetEnv\s+LM_INVENTORY_RETRIES\s+]) }
  end

  describe "managing puppetdashboard.conf with live management settings" do
    let(:file) { '/etc/puppetlabs/httpd/conf.d/puppetdashboard.conf' }
    before :each do
      @params['lm_discovery_timeout'] = 4
      @params['lm_inventory_retries'] = 3
    end

    it { should contain_file(file).with(:owner => 'root',
                                        :group => 'root',
                                        :mode => '0644') }

    it { should contain_file(file).with_content(%r[SetEnv\s+LM_DISCOVERY_TIMEOUT\s+#{@params['lm_discovery_timeout']}]) }
    it { should contain_file(file).with_content(%r[SetEnv\s+LM_INVENTORY_RETRIES\s+#{@params['lm_inventory_retries']}]) }
  end

  describe "managing passenger config" do
    let(:file) { '/etc/puppetlabs/httpd/conf.d/passenger-extra.conf' }

    it { should contain_file(file) }

    it "defaults to pool size = 4 with unknown processors" do
      @facter_facts['processorcount'] = ''
      should contain_file(file).with_content(/PassengerMaxPoolSize +4/)
    end

    # This is puppet. Anything is possible.
    it "defaults to pool size = 4 with 0 processors" do
      @facter_facts['processorcount'] = '0'
      should contain_file(file).with_content(/PassengerMaxPoolSize +4/)
    end

    it "defaults to pool size = 4 with 1 processor" do
      @facter_facts['processorcount'] = '1'
      should contain_file(file).with_content(/PassengerMaxPoolSize +4/)
    end

    it "sets pool size to 8 with 2 processors" do
      @facter_facts['processorcount'] = '2'
      should contain_file(file).with_content(/PassengerMaxPoolSize +8/)
    end

    it "sets pool size to 32 with 8 processors" do
      @facter_facts['processorcount'] = '8'
      should contain_file(file).with_content(/PassengerMaxPoolSize +32/)
    end
  end

  describe "managing license config" do
    let(:file) { '/opt/puppet/share/console/applications/licenses/config.yml' }
    it { should contain_file(file).with(:owner => 'root',
                                        :group => 'root',
                                        :mode => '0644') }

    it { should have_yaml_file(file, 'ssl' => {'cert' => "#{certdir}/dashboard-cn.cert.pem",
                                               'key' => "#{certdir}/dashboard-cn.private_key.pem",
                                               'cacert' => "/etc/puppetlabs/puppet/ssl/certs/ca.pem"}) }

    it { should have_yaml_file(file, 'puppetdb' => {'host' => 'puppetdb.rspec', 'port' => 8081}) }
  end

  describe "managing dashboard workers" do
    describe "on debian" do
      before(:each) { @facter_facts['osfamily'] = 'Debian' }

      it { should contain_file('/etc/default/pe-puppet-dashboard-workers').with(:owner => 'root',
                                                                                :group => 'root',
                                                                                :mode => '0644') }

      it { should contain_file('/etc/default/pe-puppet-dashboard-workers').with_content(/START=yes/) }

      it "defaults to 2 workers" do
        expect(subject).to contain_file('/etc/default/pe-puppet-dashboard-workers').with_content(/NUM_DELAYED_JOB_WORKERS=2/)
      end

      it "sets the specified number of workers" do
        @params['delayed_job_workers'] = 5
        expect(subject).to contain_file('/etc/default/pe-puppet-dashboard-workers').with_content(/NUM_DELAYED_JOB_WORKERS=5/)
      end
    end

    ['RedHat', 'Suse'].each do |osfamily|
      describe "on #{osfamily}" do
        before(:each) { @facter_facts['osfamily'] = osfamily }

        it { should contain_file('/etc/sysconfig/pe-puppet-dashboard-workers').with(:owner => 'root',
                                                                                    :group => 'root',
                                                                                    :mode => '0644') }

        it "defaults to 2 workers" do
          expect(subject).to contain_file('/etc/sysconfig/pe-puppet-dashboard-workers').with_content(/CPUS=2/)
        end

        it "sets the specified number of workers" do
          @params['delayed_job_workers'] = 5
          expect(subject).to contain_file('/etc/sysconfig/pe-puppet-dashboard-workers').with_content(/CPUS=5/)
        end
      end
    end
  end

  it { should contain_file('/var/log/pe-puppet-dashboard').with(:ensure => 'directory',
                                                                :owner => 'puppet-dashboard',
                                                                :group => 'puppet-dashboard',
                                                                :mode => '0644',
                                                                :recurse => true) }

  it { should contain_file('/var/log/pe-puppet-dashboard/failed_reports').with(:ensure => 'directory',
                                                                               :mode => '0600',
                                                                               :recurse => true) }

  it { should contain_file('/var/log/pe-puppet-dashboard/delayed_job.log').with(:ensure => 'present',
                                                                                :owner => 'puppet-dashboard',
                                                                                :group => 'puppet-dashboard',
                                                                                :mode => '0644') }
  describe "managing event_inspector_config.yml" do
    let(:file) { '/opt/puppet/share/event-inspector/config/config.yml' }

    it { should contain_file(file).with_content(%r[/etc/puppetlabs/puppet/ssl/certs/ca.pem]) }
    it { should contain_file(file).with_content(%r[/opt/puppet/share/puppet-dashboard/certs/dashboard-cn.private_key.pem]) }
    it { should contain_file(file).with_content(%r[/opt/puppet/share/puppet-dashboard/certs/dashboard-cn.cert.pem]) }
    it { should contain_file(file).with_content(%r[https://puppetdb.rspec:1234]) }
  end
end
